import { AmbientLight } from './ambient-light';
import { DirectionalLight } from './directional-light';
import { PointLight } from './point-light';
import { Matrix4 } from '@math.gl/core';
import type Layer from '../../lib/layer';
import type { Effect, PreRenderOptions } from '../../lib/effect';
declare type LightingEffectProps = Record<string, PointLight | DirectionalLight | AmbientLight>;
export default class LightingEffect implements Effect {
    id: string;
    props: LightingEffectProps;
    shadowColor: number[];
    private shadow;
    private ambientLight;
    private directionalLights;
    private pointLights;
    private shadowPasses;
    private shadowMaps;
    private dummyShadowMap;
    private programManager?;
    private shadowMatrices?;
    constructor(props?: LightingEffectProps);
    setProps(props: LightingEffectProps): void;
    preRender(gl: WebGLRenderingContext, { layers, layerFilter, viewports, onViewportActive, views }: PreRenderOptions): void;
    getModuleParameters(layer: Layer): {
        lightSources?: {
            ambientLight: AmbientLight | null;
            directionalLights: DirectionalLight[];
            pointLights: PointLight[];
        } | undefined;
        shadowMaps?: Texture2D[] | undefined;
        dummyShadowMap?: any;
        shadowColor?: number[] | undefined;
        shadowMatrices?: Matrix4[] | undefined;
    };
    cleanup(): void;
    private _calculateMatrices;
    private _createShadowPasses;
    private _applyDefaultLights;
}
export {};
//# sourceMappingURL=lighting-effect.d.ts.map